/*

RealIntervalTable.java
  -- classes implementing tables of bindings of strings to intervals,
     as part of the "ia_parser library" version 0.1beta1, 10/97

Copyright (C) 1997 Timothy J. Hickey

This library is free software; you can redistribute it and/or
modify it under the terms of the GNU Library General Public
License as published by the Free Software Foundation; either
version 2 of the License, or (at your option) any later version.

This library is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
Library General Public License for more details.

You should have received a copy of the GNU Library General Public
License along with this library; if not, write to the
Free Software Foundation, Inc., 59 Temple Place - Suite 330,
Boston, MA  02111-1307, USA.

 */



/*
  RealIntervalTable.java
*/

package ia_parser;

import java.lang.*;
import java.util.Vector;
import ia_math.*;

class RealIntervalEnvPair implements Cloneable{
  public String id;
  public RealInterval val;
  RealIntervalEnvPair(String i, RealInterval v) { id=i;val=v;}

  public Object clone() {
    return new RealIntervalEnvPair(id,new RealInterval(val.lo(),val.hi()));
  }

  public String toString() {
    return (" <"+id +" -> "+ val.toString()+"> ");
  }

  public String toString2() {
    return ("  "+id +" = ["+ val.lo()+",   "+val.hi()+"]");
  }

  public boolean matches(String s) {
    return(id.equals(s));
  }
}

/**
 * the RealIntervalTable class implements a class of functions from
 * Strings to RealIntervals. It provides methods for 
 * adding a (String,RealInterval) pair to the function, and for
 * looking up a RealInterval given a String.
 */
public class RealIntervalTable implements Cloneable{

  Vector environment = new Vector(10);

  /**
   * create an empty table
   */
  public RealIntervalTable() {
    super();
  }

  public RealIntervalTable(Vector v) {
    super();
    environment = v;
  }

  RealIntervalEnvPair getEnvPair(int i) {
    return (RealIntervalEnvPair) environment.elementAt(i);
  }

  public Object clone(){
    Vector e = new Vector();
    for (int i=0;i<environment.size();i++) {
      e.addElement(this.getEnvPair(i).clone());
      //        ((RealIntervalEnvPair)(environment.elementAt(i))).clone());
    }
    return new RealIntervalTable(e);
  }

  /** 
   * this returns the number of pairs in the environment
   */
  public int size() {
    return environment.size();
  }

  public String[] getNames() {
    String[] names = new String[size()];
    for (int i=0;i<names.length; i++) {
      names[i] = getEnvPair(i).id;
    }
    return names;
  }

  /**
   * Returns a string representation of the ith element of the table.
   * This is used in the Interface procedure to printout a table
   * of variable/value pairs.
   */
  public String getEnvPairString(int i) {
    return ((RealIntervalEnvPair) environment.elementAt(i)).toString();
  }

  public String getEnvPairString2(int i) {
    return 
    ((RealIntervalEnvPair) environment.elementAt(i)).toString2();
  }

  public RealInterval getNthInterval(int n) {
    return ((RealIntervalEnvPair) environment.elementAt(n)).val;
  }

  public String getNthVarName(int n) {
    return ((RealIntervalEnvPair) environment.elementAt(n)).id;
  }


  void merge(RealIntervalTable A) {
    int i;
    //                   Debugging.printTrace(Debugging.bindVarsTrace,
    //                         " merging to "+ A.environment.size());
    for(i=0;i<A.environment.size();i++){
      //                   Debugging.printTrace(Debugging.bindVarsTrace,
      //                         " merging "+ i);
      this.store(A.getEnvPair(i));
    }
  }

  /**
   * this = this intersect A
   */
  public void intersect(RealIntervalTable A) {
    RealIntervalEnvPair p;
    RealInterval var;
    String name;
    int i;
    for(i=0;i<A.environment.size();i++){
      p   =    A.getEnvPair(i);
      var = this.lookup(p.id);
      if (var == null)
        this.store(p.id,p.val);
      else {
        var.intersect(p.val);
        this.store(p.id,var);
      }
    }
  }

  /**
   * this = this union A
   */
  public void union(RealIntervalTable A) {
    RealIntervalEnvPair p;
    RealInterval var;
    String name;
    int i;
    for(i=0;i<A.environment.size();i++){
      p   =    A.getEnvPair(i);
      var = this.lookup(p.id);
      if (var == null)
        this.store(p.id,p.val);
      else {
        var.union(p.val);
        this.store(p.id,var);
      }
    }
  }

  public String toString() {
  String temp="environment(\n";
  int i;
    for(i=0;i<environment.size();i++)
      temp = temp + "  "
             +environment.elementAt(i).toString() 
             + "\n";
   temp = temp + ")\n\n";
   return(temp);
  }

  void store(RealIntervalEnvPair p) {
    this.store(p.id,p.val);
  }

  /**
   * Store the pair (s,v) in the table.
   * If a pair (s,w) is already in the table, then replace w with v.
   * Otherwise, add (s,v) as a new entry in the table.
   */
  public void store(String s, RealInterval v) {
  int i;
  RealInterval xx;
  RealIntervalEnvPair p;

    for(i=0;i<environment.size();i++) {
      p = this.getEnvPair(i);
      if (p.matches(s)) {
        p.val=v;
        return;
      }
    }
      environment.addElement(new RealIntervalEnvPair(s,v));
    return;
  }
 

  /**
   * Look for a pair (s,v) in the table.
   * If one is found, then return v.
   * Otherwise, return null.
   */
  public RealInterval lookup(String s) {
  int i;
  RealIntervalEnvPair p;
  //                 Debugging.printTrace(Debugging.evalTrace,
  //                 "  lookup:  s = " + s);

    for(i=0;i<environment.size();i++) {
      p = this.getEnvPair(i);
      //                 Debugging.printTrace(Debugging.evalTrace,
      //                 "  lookup: in loop i= " + i + " env= "+ p);
      if (p.matches(s)) {
	//                 Debugging.printTrace(Debugging.evalTrace,
	//                 "  lookup: found "+s+" = " + p.val);
        return(p.val);
      }
    }
    //                 Debugging.printTrace(Debugging.evalTrace,
    //                 "  lookup: not found "+s);
    return(null);
  }
}

  
